import { PartialRecord } from '../base';
import {
  ComputeSubnetworkConfig,
  Project,
} from '@cdktf/provider-google';
import { getKeys } from '../util';
export type Vpcs = 'infra' | 'management' | 'sales';
export type Subnets = 'db' | 'app';

type Cidr = string;
type Gateway = string;
type VpcAllocation = {
  subNets: Record<Subnets, { cidr: Cidr; gateway: Gateway }>;
};
type Topo = Record<Vpcs, VpcAllocation>;
export type Peers = PartialRecord<Vpcs, Vpcs[]>;

export type Direction = 'ingress' | 'egress'
export type SubnetConfig = PartialRecord<
  Subnets,
  { config: ComputeSubnetworkConfig }
>;

export type NetworkConfig = Record<
  Vpcs,
  {
    project: string;
    subnets: SubnetConfig;
  }
>;

export type Fw = {
  allow: boolean;
  name: string;
  priority: number;
  protocol?: string;
  sourceTags?: string[];
  sourceRanges?: string[];
  sourceNetworks?: PartialRecord<Vpcs, Subnets>
  ports?: string[];
};
type FwConfig = { ingress: Fw[]; egress: Fw[] };

export class GcpNetworkingConfig {
  readonly topo: Topo = {
    infra: {
      subNets: {
        app: {
          cidr: '10.92.0.0/22',
          gateway: '10.92.0.1',
        },
        db: {
          cidr: '10.92.4.0/24',
          gateway: '10.92.4.1',
        },
      },
    },
    management: {
      subNets: {
        app: {
          cidr: '10.93.0.0/22',
          gateway: '10.93.0.1',
        },
        db: {
          cidr: '10.93.4.0/24',
          gateway: '10.92.4.1',
        },
      },
    },
    sales: {
      subNets: {
        app: {
          cidr: '10.94.0.0/22',
          gateway: '10.94.0.1',
        },
        db: {
          cidr: '10.94.4.0/24',
          gateway: '10.94.4.1',
        },
      },
    },
  };

  readonly peers: Peers = {
    management: ['infra', 'sales'],
    infra: ['management'],
    sales: ['management'],
  };

  readonly firewall: Record<Vpcs, FwConfig> = {
    management: {
      ingress: [
        {
          name: "ping",
          allow: true,
          priority: 1000,
          protocol: 'icmp',
          sourceNetworks: {infra: 'app'}
        },
        {
          name: "catchall",
          allow: false,
          priority: 10001,
          protocol: 'all',
          sourceRanges: ["0.0.0.0/0"]
        },

      ],
      egress: [
        {
          name: "ping",
          allow: true,
          priority: 1000,
          protocol: 'icmp',
        },
        {
          name: "catchall",
          allow: false,
          priority: 10001,
          protocol: 'all',
        },
      ],
    },
    infra: {
      egress: [
        {
          name: "ping",
          allow: true,
          priority: 1000,
          protocol: 'icmp',
        },
        {
          name: "catchall",
          allow: false,
          priority: 10001,
          protocol: 'all',
        },
      ],
      ingress: [
        {
          name: "ping",
          allow: true,
          priority: 1000,
          protocol: 'icmp',
          sourceTags: ['bar'],
        },
        {
          name: "catchall",
          allow: false,
          priority: 10001,
          protocol: 'all',
          sourceRanges: ["0.0.0.0/0"]
        },
      ],
    },
    sales: {
      egress: [
        {
          name: "ping",
          allow: true,
          priority: 1000,
          protocol: 'icmp',
        },
        {
          name: "catchall",
          allow: false,
          priority: 10001,
          protocol: 'all',
        },
      ],
      ingress: [
        {
          name: "ping",
          allow: true,
          priority: 1000,
          protocol: 'icmp',
          sourceTags: ['foo'],
        },
        {
          name: "catchall",
          allow: false,
          priority: 10001,
          protocol: 'all',
          sourceRanges: ["0.0.0.0/0"]
        },
      ],
    },
  };

  config: NetworkConfig;

  constructor(project: Project) {
    const l = getKeys(this.topo).map((j) => {
      const subnets = this.topo[j].subNets;
      return {
        [j as Vpcs]: {
          project: project.name,
          subnets: getKeys(subnets)
            .map((k) => {
              return {
                [k]: {
                  config: {
                    project: project.name,
                    ipCidrRange: subnets[k].cidr,
                    gatewayAddress: subnets[k].gateway,
                  },
                },
              } as SubnetConfig;
            })
            .reduce((obj, i) => {
              return { ...obj, ...i };
            }),
        },
      } as NetworkConfig;
    });
    this.config = l.reduce((obj, i) => {
      return { ...obj, ...i };
    });
  }
}
